import tempfile
import webbrowser

from gh_util.functions import describe_latest_release
from jinja2 import Template

custom_template = Template(
    """
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{{ repo }} {{ release.name }}</title>
    <style>
        body { font-family: 'Segoe UI', Tahoma, Geneva, Verdana, sans-serif; background-color: #f4f4f4; }
        .container { max-width: 600px; margin: 20px auto; background-color: #fff; padding: 20px; border-radius: 8px; box-shadow: 0 2px 4px rgba(0,0,0,0.1); }
        h1 { color: #0056b3; }
        a { color: #007bff; text-decoration: none; }
        a:hover { text-decoration: underline; }
        .content { white-space: pre-wrap; } /* Preserves whitespace and line breaks */
    </style>
</head>
<body>
    <div class="container">
        <h1>{{ repo }} {{ release.name }}</h1>
        <p><strong>Tag:</strong> {{ release.tag_name }}</p>
        <p><strong>Published:</strong> {{ release.published_at.strftime("%A, %d %B %Y %H:%M:%S %Z") }}</p>
        <div class="content">{{ release.body }}</div>
        <p><a href="{{ release.html_url }}">View on GitHub</a></p>
    </div>
</body>
</html>
"""
)


async def main(owner: str, repo: str) -> None:
    result = await describe_latest_release(owner, repo, custom_template)

    with tempfile.NamedTemporaryFile(mode="w", suffix=".html", delete=False) as f:
        f.write(result)
        filepath = f.name

        webbrowser.open(f"file://{filepath}")


if __name__ == "__main__":
    import asyncio

    asyncio.run(main("prefecthq", "marvin"))



import httpx

import gh_util
from gh_util.logging import get_logger

logger = get_logger(__name__)


class GHClient(httpx.AsyncClient):
    """A wrapper around httpx.AsyncClient that adds GitHub authentication."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if gh_util.settings.token:
            logger.debug_kv(
                "AUTH",
                "Using GitHub token set in environment via `GH_UTIL_TOKEN`",
                "blue",
            )
            self.headers[
                "Authorization"
            ] = f"token {gh_util.settings.token.get_secret_value()}"
        else:
            logger.warning_kv(
                "AUTH",
                "`GH_UTIL_TOKEN` not set in environment - watch out for rate limits!",
                "red",
            )
        self.headers["Accept"] = "application/vnd.github.v3+json"

    async def request(self, method, url, *args, **kwargs) -> httpx.Response:
        """Allow passing in a relative URL."""
        url = str(url)
        if url.startswith("/"):
            url = f"{gh_util.settings.base_url}{url}"

        response = await super().request(method, url, *args, **kwargs)
        response.raise_for_status()
        return response



from typing import Set

from jinja2 import Template

from gh_util.client import GHClient
from gh_util.logging import get_logger
from gh_util.types import (
    GitHubComment,
    GitHubIssue,
    GitHubLabel,
    GitHubPullRequest,
    GitHubRelease,
)
from gh_util.utils import parse_as

logger = get_logger(__name__)


async def fetch_github_issue(
    owner: str,
    repo: str,
    issue_number: int,
    include_comments: bool = False,
) -> GitHubIssue:
    async with GHClient() as client:
        response = await client.get(f"/repos/{owner}/{repo}/issues/{issue_number}")
        issue = GitHubIssue.model_validate(response.json())

        if include_comments:
            response = await client.get(issue.comments_url)
            issue.user_comments = parse_as(list[GitHubComment], response.json())

        return issue


async def fetch_repo_labels(owner: str, repo: str) -> Set[GitHubLabel]:
    async with GHClient() as client:
        response = await client.get(f"/repos/{owner}/{repo}/labels")
        response.raise_for_status()
        return parse_as(set[GitHubLabel], response.json())


async def add_labels_to_issue(
    owner: str, repo: str, issue_number: int, new_labels: Set[str]
) -> bool:
    async with GHClient() as client:
        current_labels_response = await client.get(
            f"/repos/{owner}/{repo}/issues/{issue_number}/labels"
        )

        if labels_to_add := set(new_labels) - {
            label["name"] for label in current_labels_response.json()
        }:
            response = await client.post(
                f"/repos/{owner}/{repo}/issues/{issue_number}/labels",
                json=list(labels_to_add),
            )

            if response.status_code == 200:
                return True

        else:
            logger.warning_kv(
                "No change", "Selected labels already exist on issue", "blue"
            )

    return False


async def fetch_latest_release(owner: str, repo: str) -> GitHubRelease:
    async with GHClient() as client:
        response = await client.get(f"/repos/{owner}/{repo}/releases/latest")
        return GitHubRelease.model_validate(response.json())


async def describe_latest_release(
    owner: str, repo: str, template: Template | str | None = None, **render_kwargs
) -> str:
    release = await fetch_latest_release(owner, repo)
    if template is None:
        template = Template(
            "Latest release: {{ release.tag_name }} @ {{ release.html_url }}"
        )
    elif isinstance(template, str):
        template = Template(template)

    return template.render(
        release=release, **dict(owner=owner, repo=repo, **render_kwargs)
    )


async def open_pull_request(
    owner: str,
    repo: str,
    title: str,
    head: str,
    base: str,
    body: str | None = None,
    draft: bool = False,
    check_for_existing: bool = True,
) -> GitHubPullRequest:
    async with GHClient() as client:
        if check_for_existing:
            existing_prs = await client.get(
                f"/repos/{owner}/{repo}/pulls?head={owner}:{head}&base={base}&state=open"
            )
            if existing_prs.status_code == 200 and existing_prs.json():
                logger.warning_kv(
                    "Existing PR found",
                    f"PR already exists for {owner}:{head} -> {base}",
                    "blue",
                )
                return GitHubPullRequest.model_validate(existing_prs.json()[0])

        response = await client.post(
            f"/repos/{owner}/{repo}/pulls",
            json={
                "title": title,
                "head": head,
                "base": base,
                "body": body,
                "draft": draft,
            },
        )
        return GitHubPullRequest.model_validate(response.json())
